@@ -29,7 +29,7 @@ import re
 UINT64_BE = struct.Struct(">Q")
 UINT32 = struct.Struct("<I")
 UINT32_BE = struct.Struct(">I")
-USHORT = struct.Struct("<H")
+UINT16 = struct.Struct("<H")
 
 TBPIECES = 7
 
@@ -604,26 +604,26 @@ class Table(object):
         if self.data is None:
             self.data = mmap.mmap(self.fd, 0, access=mmap.ACCESS_READ)
 
-            # Micro optimizations for read_ubyte.
+            # Micro optimizations for read_uint8.
             if sys.version_info >= (3, ):
-                self.read_ubyte = self.data.__getitem__
+                self.read_uint8 = self.data.__getitem__
             else:
-                def read_ubyte(data_ptr):
+                def read_uint8(data_ptr):
                     return ord(self.data[data_ptr])
 
-                self.read_ubyte = read_ubyte
+                self.read_uint8 = read_uint8
 
     def check_magic(self, magic):
-        return all(self.read_ubyte(i) == m for i, m in enumerate(magic))
+        return all(self.read_uint8(i) == m for i, m in enumerate(magic))
 
     def setup_pairs(self, data_ptr, tb_size, size_idx, wdl):
         d = PairsData()
 
-        self._flags = self.read_ubyte(data_ptr)
-        if self.read_ubyte(data_ptr) & 0x80:
+        self._flags = self.read_uint8(data_ptr)
+        if self.read_uint8(data_ptr) & 0x80:
             d.idxbits = 0
             if wdl:
-                d.min_len = self.read_ubyte(data_ptr + 1)
+                d.min_len = self.read_uint8(data_ptr + 1)
             else:
                 # http://www.talkchess.com/forum/viewtopic.php?p=698093#698093
                 d.min_len = 1 if self.variant.captures_compulsory else 0
@@ -633,15 +633,15 @@ class Table(object):
             self.size[size_idx + 2] = 0
             return d
 
-        d.blocksize = self.read_ubyte(data_ptr + 1)
-        d.idxbits = self.read_ubyte(data_ptr + 2)
+        d.blocksize = self.read_uint8(data_ptr + 1)
+        d.idxbits = self.read_uint8(data_ptr + 2)
 
         real_num_blocks = self.read_uint32(data_ptr + 4)
-        num_blocks = real_num_blocks + self.read_ubyte(data_ptr + 3)
-        max_len = self.read_ubyte(data_ptr + 8)
-        min_len = self.read_ubyte(data_ptr + 9)
+        num_blocks = real_num_blocks + self.read_uint8(data_ptr + 3)
+        max_len = self.read_uint8(data_ptr + 8)
+        min_len = self.read_uint8(data_ptr + 9)
         h = max_len - min_len + 1
-        num_syms = self.read_ushort(data_ptr + 10 + 2 * h)
+        num_syms = self.read_uint16(data_ptr + 10 + 2 * h)
 
         d.offset = data_ptr + 10
         d.symlen = [0 for _ in range(h * 8 + num_syms)]
@@ -663,7 +663,7 @@ class Table(object):
         d.base = [0 for _ in range(h)]
         d.base[h - 1] = 0
         for i in range(h - 2, -1, -1):
-            d.base[i] = (d.base[i + 1] + self.read_ushort(d.offset + i * 2) - self.read_ushort(d.offset + i * 2 + 2)) // 2
+            d.base[i] = (d.base[i + 1] + self.read_uint16(d.offset + i * 2) - self.read_uint16(d.offset + i * 2 + 2)) // 2
         for i in range(h):
             d.base[i] <<= 64 - (min_len + i)
 
@@ -753,11 +753,11 @@ class Table(object):
 
     def calc_symlen(self, d, s, tmp):
         w = d.sympat + 3 * s
-        s2 = (self.read_ubyte(w + 2) << 4) | (self.read_ubyte(w + 1) >> 4)
+        s2 = (self.read_uint8(w + 2) << 4) | (self.read_uint8(w + 1) >> 4)
         if s2 == 0x0fff:
             d.symlen[s] = 0
         else:
-            s1 = ((self.read_ubyte(w + 1) & 0xf) << 8) | self.read_ubyte(w)
+            s1 = ((self.read_uint8(w + 1) & 0xf) << 8) | self.read_uint8(w)
             if not tmp[s1]:
                 self.calc_symlen(d, s1, tmp)
             if not tmp[s2]:
@@ -949,16 +949,16 @@ class Table(object):
         litidx = (idx & (1 << d.idxbits) - 1) - (1 << (d.idxbits - 1))
         block = self.read_uint32(d.indextable + 6 * mainidx)
 
-        idx_offset = self.read_ushort(d.indextable + 6 * mainidx + 4)
+        idx_offset = self.read_uint16(d.indextable + 6 * mainidx + 4)
         litidx += idx_offset
 
         if litidx < 0:
             while litidx < 0:
                 block -= 1
-                litidx += self.read_ushort(d.sizetable + 2 * block) + 1
+                litidx += self.read_uint16(d.sizetable + 2 * block) + 1
         else:
-            while litidx > self.read_ushort(d.sizetable + 2 * block):
-                litidx -= self.read_ushort(d.sizetable + 2 * block) + 1
+            while litidx > self.read_uint16(d.sizetable + 2 * block):
+                litidx -= self.read_uint16(d.sizetable + 2 * block) + 1
                 block += 1
 
         ptr = d.data + (block << d.blocksize)
@@ -975,7 +975,7 @@ class Table(object):
             l = m
             while code < d.base[base_idx + l]:
                 l += 1
-            sym = self.read_ushort(d.offset + l * 2)
+            sym = self.read_uint16(d.offset + l * 2)
             sym += (code - d.base[base_idx + l]) >> (64 - l)
             if litidx < d.symlen[symlen_idx + sym] + 1:
                 break
@@ -993,14 +993,18 @@ class Table(object):
         sympat = d.sympat
         while d.symlen[symlen_idx + sym]:
             w = sympat + 3 * sym
-            s1 = ((self.read_ubyte(w + 1) & 0xf) << 8) | self.read_ubyte(w)
+            s1 = ((self.read_uint8(w + 1) & 0xf) << 8) | self.read_uint8(w)
             if litidx < d.symlen[symlen_idx + s1] + 1:
                 sym = s1
             else:
                 litidx -= d.symlen[symlen_idx + s1] + 1
-                sym = (self.read_ubyte(w + 2) << 4) | (self.read_ubyte(w + 1) >> 4)
+                sym = (self.read_uint8(w + 2) << 4) | (self.read_uint8(w + 1) >> 4)
 
-        return self.read_ubyte(sympat + 3 * sym)
+        w = sympat + 3 * sym
+        if isinstance(self, DtzTable):
+            return ((self.read_uint8(w + 1) & 0x0f) << 8) | self.read_uint8(w)
+        else:
+            return self.read_uint8(w)
 
     def read_uint64_be(self, data_ptr):
         return UINT64_BE.unpack_from(self.data, data_ptr)[0]
@@ -1011,8 +1015,8 @@ class Table(object):
     def read_uint32_be(self, data_ptr):
         return UINT32_BE.unpack_from(self.data, data_ptr)[0]
 
-    def read_ushort(self, data_ptr):
-        return USHORT.unpack_from(self.data, data_ptr)[0]
+    def read_uint16(self, data_ptr):
+        return UINT16.unpack_from(self.data, data_ptr)[0]
 
     def close(self):
         if self.data is not None:
@@ -1069,8 +1073,8 @@ class WdlTable(Table):
             self._flags = None
             self.flags = None
 
-            split = self.read_ubyte(4) & 0x01
-            files = 4 if self.read_ubyte(4) & 0x02 else 1
+            split = self.read_uint8(4) & 0x01
+            files = 4 if self.read_uint8(4) & 0x02 else 1
 
             data_ptr = 5
 
@@ -1151,30 +1155,30 @@ class WdlTable(Table):
 
     def setup_pieces_pawn(self, p_data, p_tb_size, f):
         j = 1 + int(self.pawns[1] > 0)
-        order = self.read_ubyte(p_data) & 0x0f
-        order2 = self.read_ubyte(p_data + 1) & 0x0f if self.pawns[1] else 0x0f
-        self.files[f].pieces[0] = [self.read_ubyte(p_data + i + j) & 0x0f for i in range(self.num)]
+        order = self.read_uint8(p_data) & 0x0f
+        order2 = self.read_uint8(p_data + 1) & 0x0f if self.pawns[1] else 0x0f
+        self.files[f].pieces[0] = [self.read_uint8(p_data + i + j) & 0x0f for i in range(self.num)]
         self.files[f].norm[0] = [0 for _ in range(self.num)]
         self.set_norm_pawn(self.files[f].norm[0], self.files[f].pieces[0])
         self.files[f].factor[0] = [0 for _ in range(TBPIECES)]
         self.tb_size[p_tb_size] = self.calc_factors_pawn(self.files[f].factor[0], order, order2, self.files[f].norm[0], f)
 
-        order = self.read_ubyte(p_data) >> 4
-        order2 = self.read_ubyte(p_data + 1) >> 4 if self.pawns[1] else 0x0f
-        self.files[f].pieces[1] = [self.read_ubyte(p_data + i + j) >> 4 for i in range(self.num)]
+        order = self.read_uint8(p_data) >> 4
+        order2 = self.read_uint8(p_data + 1) >> 4 if self.pawns[1] else 0x0f
+        self.files[f].pieces[1] = [self.read_uint8(p_data + i + j) >> 4 for i in range(self.num)]
         self.files[f].norm[1] = [0 for _ in range(self.num)]
         self.set_norm_pawn(self.files[f].norm[1], self.files[f].pieces[1])
         self.files[f].factor[1] = [0 for _ in range(TBPIECES)]
         self.tb_size[p_tb_size + 1] = self.calc_factors_pawn(self.files[f].factor[1], order, order2, self.files[f].norm[1], f)
 
     def setup_pieces_piece(self, p_data):
-        self.pieces[0] = [self.read_ubyte(p_data + i + 1) & 0x0f for i in range(self.num)]
-        order = self.read_ubyte(p_data) & 0x0f
+        self.pieces[0] = [self.read_uint8(p_data + i + 1) & 0x0f for i in range(self.num)]
+        order = self.read_uint8(p_data) & 0x0f
         self.set_norm_piece(self.norm[0], self.pieces[0])
         self.tb_size[0] = self.calc_factors_piece(self.factor[0], order, self.norm[0])
 
-        self.pieces[1] = [self.read_ubyte(p_data + i + 1) >> 4 for i in range(self.num)]
-        order = self.read_ubyte(p_data) >> 4
+        self.pieces[1] = [self.read_uint8(p_data + i + 1) >> 4 for i in range(self.num)]
+        order = self.read_uint8(p_data) >> 4
         self.set_norm_piece(self.norm[1], self.pieces[1])
         self.tb_size[1] = self.calc_factors_piece(self.factor[1], order, self.norm[1])
 
@@ -1262,7 +1266,7 @@ class DtzTable(Table):
             self.size = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
             self.files = [PawnFileDataDtz() for f in range(4)]
 
-            files = 4 if self.read_ubyte(4) & 0x02 else 1
+            files = 4 if self.read_uint8(4) & 0x02 else 1
 
             p_data = 5
 
@@ -1278,10 +1282,15 @@ class DtzTable(Table):
                 p_data = self._next
                 self.p_map = p_data
                 if self.flags & 2:
-                    for i in range(4):
-                        self.map_idx[i] = p_data + 1 - self.p_map
-                        p_data += 1 + self.read_ubyte(p_data)
-                    p_data += p_data & 0x01
+                    if not self.flags & 16:
+                        for i in range(4):
+                            self.map_idx[i] = p_data + 1 - self.p_map
+                            p_data += 1 + self.read_uint8(p_data)
+                    else:
+                        for i in range(4):
+                            self.map_idx[i] = (p_data + 2 - self.p_map) // 2
+                            p_data += 2 + 2 * self.read_uint16(p_data)
+                p_data += p_data & 0x01
 
                 self.precomp.indextable = p_data
                 p_data += self.size[0]
@@ -1313,9 +1322,15 @@ class DtzTable(Table):
                 for f in range(files):
                     self.map_idx.append([])
                     if self.flags[f] & 2:
-                        for i in range(4):
-                            self.map_idx[-1].append(p_data + 1 - self.p_map)
-                            p_data += 1 + self.read_ubyte(p_data)
+                        if not self.flags[f] & 16:
+                            for _ in range(4):
+                                self.map_idx[-1].append(p_data + 1 - self.p_map)
+                                p_data += 1 + self.read_uint8(p_data)
+                        else:
+                            p_data += p_data & 0x01
+                            for _ in range(4):
+                                self.map_idx[-1].append((p_data + 2 - self.p_map) // 2)
+                                p_data += 2 * 2 + self.read_uint16(p_data)
                 p_data += p_data & 0x01
 
                 for f in range(files):
@@ -1371,7 +1386,10 @@ class DtzTable(Table):
             res = self.decompress_pairs(self.precomp, idx)
 
             if self.flags & 2:
-                res = self.read_ubyte(self.p_map + self.map_idx[WDL_TO_MAP[wdl + 2]] + res)
+                if not self.flags & 16:
+                    res = self.read_uint8(self.p_map + self.map_idx[WDL_TO_MAP[wdl + 2]] + res)
+                else:
+                    res = self.read_uint16(self.p_map + 2 * (self.map_idx[WDL_TO_MAP[wdl + 2]] + res))
 
             if (not (self.flags & PA_FLAGS[wdl + 2])) or (wdl & 1):
                 res *= 2
@@ -1404,7 +1422,10 @@ class DtzTable(Table):
             res = self.decompress_pairs(self.files[f].precomp, idx)
 
             if self.flags[f] & 2:
-                res = self.read_ubyte(self.p_map + self.map_idx[f][WDL_TO_MAP[wdl + 2]] + res)
+                if not self.flags[f] & 16:
+                    res = self.read_uint8(self.p_map + self.map_idx[f][WDL_TO_MAP[wdl + 2]] + res)
+                else:
+                    res = self.read_uint16(self.p_map + 2 * (self.map_idx[f][WDL_TO_MAP[wdl + 2]] + res))
 
             if (not (self.flags[f] & PA_FLAGS[wdl + 2])) or (wdl & 1):
                 res *= 2
@@ -1412,16 +1433,16 @@ class DtzTable(Table):
         return res, 1
 
     def setup_pieces_piece_dtz(self, p_data, p_tb_size):
-        self.pieces = [self.read_ubyte(p_data + i + 1) & 0x0f for i in range(self.num)]
-        order = self.read_ubyte(p_data) & 0x0f
+        self.pieces = [self.read_uint8(p_data + i + 1) & 0x0f for i in range(self.num)]
+        order = self.read_uint8(p_data) & 0x0f
         self.set_norm_piece(self.norm, self.pieces)
         self.tb_size[p_tb_size] = self.calc_factors_piece(self.factor, order, self.norm)
 
     def setup_pieces_pawn_dtz(self, p_data, p_tb_size, f):
         j = 1 + int(self.pawns[1] > 0)
-        order = self.read_ubyte(p_data) & 0x0f
-        order2 = self.read_ubyte(p_data + 1) & 0x0f if self.pawns[1] else 0x0f
-        self.files[f].pieces = [self.read_ubyte(p_data + i + j) & 0x0f for i in range(self.num)]
+        order = self.read_uint8(p_data) & 0x0f
+        order2 = self.read_uint8(p_data + 1) & 0x0f if self.pawns[1] else 0x0f
+        self.files[f].pieces = [self.read_uint8(p_data + i + j) & 0x0f for i in range(self.num)]
 
         self.files[f].norm = [0 for _ in range(self.num)]
         self.set_norm_pawn(self.files[f].norm, self.files[f].pieces)
