@@ -38,6 +38,18 @@ DUMMY_RESPONSES = [ENGINE_RESIGN, GAME_DRAW] = [-1, -2]
 RESULTS = [WHITE_WIN, BLACK_WIN, DRAW] = ["1-0", "0-1", "1/2-1/2"]
 
 
+def try_move(board, move):
+    try:
+        move = board.push_uci(move)
+    except ValueError:
+        try:
+            move = board.push_san(move)
+        except ValueError:
+            LOGGER.exception("exception parsing pv")
+            return None
+    return move
+
+
 class DrawHandler(object):
     """
     Chess engines may send a draw offer after playing its move and may receive
@@ -311,6 +323,9 @@ class Engine(object):
         self.move = None
         self.move_received = threading.Event()
 
+        self.ponder_on = None
+        self.ponder_move = None
+
         self.return_code = None
         self.terminated = threading.Event()
 
@@ -361,6 +376,8 @@ class Engine(object):
                 return self._move(command_and_args[1])
             elif command_and_args[0] == "offer" and command_and_args[1] == "draw":
                 return self._offer_draw()
+            elif command_and_args[0] == "Hint:":
+                return self._hint(command_and_args[1])
         elif len(command_and_args) >= 5:
             return self._post(buf)
 
@@ -468,6 +485,12 @@ class Engine(object):
         for post_handler in self.post_handlers:
             post_handler.on_move(self.move)
 
+    def _hint(self, arg):
+        # If we have finished search and received a best move,
+        # the Hint tells us the ponder move for supported engines
+        if self.move_received.is_set():
+            self.ponder_move = arg
+
     def _post(self, arg):
         if not self.post_handlers:
             return
@@ -488,6 +511,21 @@ class Engine(object):
 
         pv = []
         board = self.board.copy(stack=False)
+
+        # Ponder may be handled in one (or both) of two ways according to the
+        # spec. Either through a 'Hint: <move>' or through '5. ... (<move>) pv'.
+        # It is unclear whether the 'Hint: <move>' variation is persistent
+        # until changed or whether it must be given before each ponder post.
+
+        # Assumption: The hint ponder overrides the pv ponder.
+        #             They should be the same in a normal scenario.
+
+        making_pv_ponder = False # For the '(<move>)' variation
+        hint_ponder_played = False # For the 'Hint: <move>' variation
+        if self.ponder_move:
+            try_move(board, self.ponder_move)
+            hint_ponder_played = True
+
         tokens = arg.split()
         # Order: <score> <depth> <time> <nodes> <pv>.
         handle_integer_token(tokens[0], lambda handler, val: handler.depth(val))
@@ -498,13 +536,24 @@ class Engine(object):
             # Ignore move number. For example, 1. Nf3 Nf6 -> Nf3 Nf6.
             if '.' in token or '<' in token:
                 continue
-            try:
-                pv.append(board.push_uci(token))
-            except ValueError:
-                try:
-                    pv.append(board.push_san(token))
-                except ValueError:
-                    LOGGER.exception("exception parsing pv")
+            # Handle ponder move only if we haven't already received a hint
+            elif token.startswith('(') and token.endswith(')'):
+                # If a Hint was given and the ponder move played,
+                # ignore the move in parantheses
+                if hint_ponder_played:
+                    continue
+                token = token[1:-1]
+                self.ponder_move = token
+                making_pv_ponder = True
+
+            pv_move = try_move(board, token)
+
+            # Don't append to pv if this move was the ponder move
+            if making_pv_ponder:
+                making_pv_ponder = False
+                continue
+
+            pv.append(pv_move)
 
         if pv is not None:
             for post_handler in self.post_handlers:
@@ -635,6 +684,8 @@ class Engine(object):
 
         :return: Nothing.
         """
+        self.ponder_on = ponder
+
         if ponder:
             msg = "hard"
         else:
@@ -1207,13 +1258,7 @@ class Engine(object):
             if self.move in DUMMY_RESPONSES:
                 return self.move
 
-            try:
-                self.board.push_uci(str(self.move))
-            except ValueError:
-                try:
-                    self.board.push_san(str(self.move))
-                except ValueError:
-                    LOGGER.exception("exception parsing move")
+            try_move(self.board, str(self.move))
 
             return self.move
 
@@ -1284,19 +1329,13 @@ class Engine(object):
             for post_handler in self.post_handlers:
                 post_handler.on_go()
 
-        try:
-            self.board.push_uci(str(move))
-        except ValueError:
-            try:
-                self.board.push_san(str(move))
-            except ValueError:
-                LOGGER.exception("exception parsing move")
+        try_move(self.board, str(move))
 
         builder.append(str(move))
 
         def command():
-            # Use the join(builder) once we parse usermove=1 feature.
             move_str = " ".join(builder)
+            self.ponder_move = None
             if self.in_force:
                 with self.semaphore:
                     self.send_line(move_str)
@@ -1317,13 +1356,7 @@ class Engine(object):
                 if self.terminated.is_set():
                     raise EngineTerminatedException()
 
-                try:
-                    self.board.push_uci(str(self.move))
-                except ValueError:
-                    try:
-                        self.board.push_san(str(self.move))
-                    except ValueError:
-                        LOGGER.exception("exception parsing move")
+                try_move(self.board, str(self.move))
 
                 return self.move
 
